** Author: Stphane DESNAULT - SOFTWARE ASSETS
** 
** 12 rue Franois Grard
** 75016 PARIS - FRANCE
** 
** Tel : (331) 40 50 39 40
** Fax : (331) 40 50 39 50
** CIS : 100034,2567
** 
** If you like SYMAEDIT and need database expertise... give us a call!
**
****************************************************************

FUNCTION symAEdit

PARAMETERS laArr,laTitles,lcPicture, lnVCnt, lnHCnt

* laArr			: Array to edit of dimension [N(N+1)/2]
* laTitles  : Array of titles of dimension [N]
* lcPicture : Picture of edited items
* lnVCnt		: Number of items to display vertically - optional
* lnHCnt		: Number of items to display horizontally - optional

* Format for laArr
* ----------------
*	Foxpro doesn't give us any easy way to store symmetric matrices,
* i.e. arrays for which, for any given i,j laArr[i,j]=laArr[j,i].
*
* Symetric arrays are common. When working with databases we find
* them often enough each time we model N-to-N relationships.
*
* laArr stores a symmetric array ROW WISE, starting with the
* smaller rows. In other words :
*
* laArr=[1,2,3,4,5,6] represents the 3x3 array symArr:
*
*   
* \  1  2  3 
*  +--------- 
* 1! 1 
*  !
* 2! 2  3
*  !
* 3! 4  5  6
*
* Now, how do I go back and forth beetween symArr's [i,j] indexes 
* and the unique index of laArr?
*
* You can check that symArr[i,j] is represented by element
* number j+(i*(i-1))/2 of laArr.
*
*	Inversely, given x the index in laArr, we have:
*
* i=CEILING((SQRT(8*x+1)-1)/2)
* j=x-(i*(i-1)/2)
 


PRIVATE lnTopRow							&& Index of top-most displayed row
PRIVATE lnLeftCol							&& Index of left-most displayed column
PRIVATE lnRow									&& CURSOR Row of current cell
PRIVATE lnCol									&& CURSOR Column of current cell
PRIVATE lnArrDim							&& Dimension of array laArr
PRIVATE	i,j										&& i loops on ROWS, j loops on COLUMNS
PRIVATE lnCellLen							&& Length of a cell
PRIVATE lnLeftT								&& Character Length of Title column
PRIVATE lnRowLen							&& Length of a displayed row (including titles)
PRIVATE llExit								&& Signals exit from the main loop
PRIVATE lnHElev								&& Position inside horizontal elevator
PRIVATE lnVElev								&& position inside vertical elevator
PRIVATE lcStatus							&& Status on exit


IF SET("TALK") = "ON"
	SET TALK OFF
	lcTalkStat = "ON"
ELSE
	lcTalkStat = "OFF"
ENDIF

lcCompStat = SET("COMPATIBLE")
SET COMPATIBLE FOXPLUS

=ACOPY(laArr,laBackup)

lnArrDim=(SQRT(8*ALEN(laArr,1)+1)-1)/2

lnCellLen=LEN(TRANSFORM(laArr[1],lcPicture))+1

* Scan laTitles for the longest title
lnLeftT=0
FOR i=1 TO ALEN(laTitles,1)
	IF lnLeftT < LEN(ALLTRIM(laTitles[i]))
		lnLeftT=LEN(ALLTRIM(laTitles[i]))
	ENDIF
ENDFOR

* Compute displayed line length. Line length is either computed
* after lnHCnt if it's present or after the screen's width. If
* lnHCnt is present but too big to fit the screen it is reduced
* accordingly, ditto if the dipslayed array doesn't have that
* many columns.

IF (PARAMETERS()<6) .OR. (lnHCnt>lnArrDim)
	 lnHCnt=lnArrDim
ENDIF


lnRowLen=lnLeftT+lnHCnt*lnCellLen
IF lnRowLen>(SCOLS()-6)
	lnHCnt=FLOOR((SCOLS()-6-lnLeftT)/lnCellLen)
	lnRowLen=lnLeftT+lnHCnt*lnCellLen
ENDIF

* Compute number of Rows displayed.
IF (PARAMETERS()<5) .OR. (lnVCnt>lnArrDim)
	lnVCnt=lnArrDim
ENDIF

IF lnVCnt>(SROWS()-9)
	lnVCnt=(SROWS()-9)
ENDIF

DEFINE WINDOW editArray ;
		FROM INT((SROW()-(lnVCnt+9))/2),INT((SCOL()-(lnRowLen+6))/2) ;
		TO INT((SROW()-(lnVCnt+9))/2)+(lnVCnt+8),INT((SCOL()-(lnRowLen+6))/2)+(lnRowLen+5) ;
		FLOAT ;
		NOCLOSE ;
		DOUBLE ;
		COLOR SCHEME 10

ACTIVATE WINDOW editarray

lnTopRow=1
lnRow=1
lnLeftCol=1
lnCol=1
llExit=.F.
lnHElev=1
lnVElev=1
lcStatus="OK"

* Draw background lines
@ 2,1 TO 2,lnRowLen+1
@ lnVcnt+3,1 TO lnVcnt+3,IIF(lnHCnt<lnArrDim,lnLeftT,lnRowLen+1)

* Draw exit buttons
@ lnVCnt+5,WCOLS()-20 SAY "  OK    <Cancel>"

=redraw("TOP+LEFT")
=drawCursor()

DO WHILE ! llExit

	lnKeyPress=INKEY("M")

	DO CASE
		CASE lnKeypress=4																	&& Right
			=moveRight()
		CASE lnKeypress=19 OR lnKeyPress=127							&& Left/BackSpace
			=moveLeft()
		CASE lnKeypress=24																&& Down
			=moveDown()
		CASE lnKeypress=5																	&& Up
			=moveUp()
		CASE lnKeypress=1																	&& Home
			=moveHome()
		CASE lnKeypress=6																	&& End
			=moveEnd()
		CASE lnKeypress=9																	&& Tab
			=screenRight()
		CASE lnKeypress=15																&& Backtab
			=screenLeft()
		CASE lnKeypress=18																&& Page up
			=screenUp()
		CASE lnKeypress=3																	&& Page down
			=screenDown()
		CASE lnKeypress=13																&& Enter
			=editCell()
		CASE lnKeypress=10																&& Ctrl-Enter
			=exitAEdit("OK")
		CASE lnKeypress=27																&& Escape
			=exitAEdit("Cancel")
		CASE lnKeypress=151																&& Mouse
			=mousePress()
		CASE BETWEEN(lnKeypress,48,57) OR lnKeypress=45		&& "-" or digit
			SET CURSOR ON
			@ lnRow+2,lnLeftT+2+(lnCol-1)*lnCellLen GET laArr[ijTOx(lnRow+lnTopRow-1,lnCol+lnLeftCol-1)] PICTURE lcPicture
			KEYBOARD CHR(lnKeypress)
			READ
			SET CURSOR OFF
			=drawCursor()
		CASE lnKeypress=7											&& Delete
			laArr[ijTOx(lnTopRow+lnRow-1,lnLeftCol+lnCol-1)]=0
			@ lnRow+2,lnLeftT+2+(lnCol-1)*lnCellLen SAY 0 PICTURE lcPicture
			=drawCursor()
	ENDCASE
ENDDO

RELEASE WINDOW editArray

IF lcStatus="Cancel"
	=ACOPY(laBackup,laArr)
ENDIF

SET TALK &lcTalkStat
SET COMPATIBLE &lcCompStat

RETURN lcStatus


****************************************************************


****************************************************************

FUNCTION ijTOx

PARAMETERS i,j

RETURN j+i*(i-1)/2

****************************************************************


****************************************************************

FUNCTION redraw

PARAMETERS lcTitles

* Draw Col titles at the top
IF 'TOP' $ UPPER(lcTitles)
	FOR j=1 TO lnHCnt
		@ 1,lnLeftT+2+(j-1)*lnCellLen SAY PADC(ALLTRIM(laTitles[j+lnLeftCol-1]),lnCellLen)
	ENDFOR
	=drawElev("H")
ENDIF

* Draw Row titles on the left
IF 'LEFT' $ UPPER(lcTitles)
	FOR i=1 TO lnVCnt
		@ i+2,1 SAY PADR(laTitles[i+lnTopRow-1],lnLeftT)
	ENDFOR
	=drawElev("V")
ENDIF

* Draw cells
FOR i=1 TO lnVCnt
	FOR j=1 TO lnHCnt
		IF (lnTopRow+i-1)>=(lnLeftCol+j-1)
			@ i+2,lnLeftT+2+(j-1)*lnCellLen SAY laArr[ijTOx(lnTopRow+i-1,lnLeftCol+j-1)] PICTURE lcPicture
		ELSE
			@ i+2,lnLeftT+2+(j-1)*lnCellLen SAY SPACE(lnCellLen)
		ENDIF
	ENDFOR
ENDFOR

RETURN .T.

****************************************************************


****************************************************************

FUNCTION drawCursor

* Column Title
@ 1,lnLeftT+2+(lnCol-1)*lnCellLen FILL TO ;
	1,lnLeftT+lnCol*lnCellLen+1 COLOR (SCHEME(10,7))

* Row title
@ lnRow+2,1 FILL TO lnRow+2,1+lnLeftT COLOR (SCHEME(10,7))

* Cell
@ lnRow+2,lnLeftT+2+(lnCol-1)*lnCellLen FILL TO ;
	lnRow+2,lnLeftT+lnCol*lnCellLen+1 COLOR (SCHEME(10,2))

RETURN .T.

****************************************************************


****************************************************************

FUNCTION drawElev

PARAMETERS lcHV

* If horizontal elevator is requested...
IF 'H' $ UPPER(lcHV)
	* ...and if it is needed...
	IF lnHCnt<lnArrDim
		* ...Draw it!
		lnHElev=(lnLeftCol+lnCol-1)*(lnCellLen*lnHCnt-1)/lnArrDim
		@ lnVCnt+3,lnLeftT+1 SAY ""+SPACE(lnHCnt*lnCellLen-1)+"" ;
			COLOR (SCHEME(10,3))
		@ lnVCnt+3,lnLeftT+1+lnHelev SAY "" COLOR (SCHEME(10,3))
	ENDIF
ENDIF

* If vertical elevator is requested...
IF 'V' $ UPPER(lcHV)
	* ...and if it is needed...
	IF lnVCnt<lnArrDim
		* ...Draw it!
		lnVElev=(lnTopRow+lnRow-1)*(lnVCnt-3)/lnArrDim
		@ 2,lnRowLen+2 SAY "" COLOR (SCHEME(10,3))
		@ 3,lnRowLen+2 SAY "" COLOR (SCHEME(10,3))
		@ 4,lnRowLen+2 CLEAR TO lnVcnt+1,lnRowLen+2
		@ 4,lnRowLen+2 FILL TO lnVcnt+1,lnRowLen+2 COLOR (SCHEME(10,3))
		@ lnVCnt+2,lnRowLen+2 SAY "" COLOR (SCHEME(10,3))
		@ lnVCnt+3,lnRowLen+2 SAY IIF(lnHCnt<lnArrDim," ","") ;
			COLOR (SCHEME(10,3))
		@ lnVElev+4,lnRowLen+2 SAY "" COLOR (SCHEME(10,3))
	ENDIF
ENDIF

RETURN .T.

****************************************************************


****************************************************************

FUNCTION clearCursor

* Column Title
@ 1,lnLeftT+2+(lnCol-1)*lnCellLen FILL TO ;
	1,lnLeftT+lnCol*lnCellLen+1 COLOR (SCHEME(10,1))

* Row title
@ lnRow+2,1 FILL TO lnRow+2,1+lnLeftT COLOR (SCHEME(10,1))

* Cell
@ lnRow+2,lnLeftT+2+(lnCol-1)*lnCellLen FILL TO ;
	lnRow+2,lnLeftT+lnCol*lnCellLen+1 COLOR (SCHEME(10,1))

RETURN .T.

****************************************************************


****************************************************************

FUNCTION editCell

SET CURSOR ON
@ lnRow+2,lnLeftT+2+(lnCol-1)*lnCellLen GET laArr[ijTOx(lnRow+lnTopRow-1,lnCol+lnLeftCol-1)] PICTURE lcPicture
READ
SET CURSOR OFF
=drawCursor()

RETURN .T.

****************************************************************


****************************************************************

FUNCTION moveLeft

* If cursor was on the left of the screen...
IF lnCol=1
	*...Then if it's not on the left of the array...
	IF (lnLeftCol>1)
		* ...move the window one column to the left,
		* leaving cursor in place.
		lnLeftCol=lnLeftCol-1
		=redraw("TOP")
		=drawCursor()
	ENDIF
ELSE
	*...Otherwise, simply move cursor to the right.
	=ClearCursor()
	lnCol=lnCol-1
	=drawCursor()
	=drawElev("H")
ENDIF

RETURN .T.

****************************************************************


****************************************************************

FUNCTION moveRight

PRIVATE lcRedraw,lcMove

lcRedraw=""
lcMove="H"
* If we are not at the very end of the array...
IF (lnTopRow+lnRow-1)<lnArrDim OR (lnLeftCol+lnCol-1)<lnArrDim
	=clearCursor()
	* If the cursor is on the diagonal...
	IF lnTopRow+lnRow-1=lnLeftCol+lnCol-1
		* ...move down one row.
		lcMove=lcMove+"V"
		IF lnRow=lnVCnt
			lcRedraw="LEFT"
			lnTopRow=lnTopRow+1
		ELSE
			lnRow=lnRow+1
		ENDIF
	ENDIF
	* If cursor was on the right of the screen...
	IF lnCol=lnHCnt
		* ...move the window one column right,
		* leaving cursor in place.
		lnLeftCol=lnLeftCol+1
		lcRedraw=lcRedraw+"TOP"
	ELSE
		*...Otherwise, simply move cursor right.
		lnCol=lnCol+1
	ENDIF
	IF ! EMPTY(lcRedraw)
		=redraw(lcRedraw)
	ENDIF
	=drawCursor()
	=drawElev(lcMove)

ENDIF

RETURN .T.

****************************************************************


****************************************************************

FUNCTION moveUp

PRIVATE lcRedraw,lcMove

lcRedraw=""
lcMove="V"
* If we are not at the very beginning of the array...
IF lnTopRow+lnRow-1>1
	=clearCursor()
	* If the cursor is on the diagonal...
	IF lnTopRow+lnRow-1=lnLeftCol+lnCol-1
		* ...move left one row.
		lcMove=lcMove+"H"
		IF lnCol=1
			lcRedraw="TOP"
			lnLeftCol=lnLeftCol-1
		ELSE
			lnCol=lnCol-1
		ENDIF
	ENDIF
	* If cursor was on top of the screen...
	IF lnRow=1
		* ...move the window one row up,
		* leaving cursor in place.
		lnTopRow=lnTopRow-1
		lcRedraw=lcRedraw+"LEFT"
	ELSE
		*...Otherwise, simply move cursor up.
		lnRow=lnRow-1
	ENDIF
	IF ! EMPTY(lcRedraw)
		=redraw(lcRedraw)
	ENDIF
	=drawCursor()
	=drawElev(lcMove)

ENDIF

RETURN .T.

****************************************************************


****************************************************************

FUNCTION moveDown

* If cursor was on the bottom of the screen...
IF lnRow=lnVCnt
	*...Then if it's not on the bottom of the array...
	IF (lnTopRow+lnVCnt-1)<lnArrDim
		* ...move the window one line down,
		* leaving cursor in place.
		lnTopRow=lnTopRow+1
		=redraw("LEFT")
		=drawCursor()
	ENDIF
ELSE
	*...Otherwise, simply move cursor down.
	=ClearCursor()
	lnRow=lnRow+1
	=drawCursor()
	=drawElev("V")
ENDIF

RETURN .T.

****************************************************************


****************************************************************

FUNCTION moveHome

PRIVATE lcTemp

lctemp=""               
IF lnLeftCol>1          
  lcTemp="TOP"          
ENDIF                   
IF lnTopRow>1           
  lcTemp=lcTemp+"LEFT"  
ENDIF                   
=ClearCursor()          
lnTopRow=1              
lnLeftCol=1             
lnRow=1                 
lnCol=1                 
IF ! EMPTY(lcTemp)      
  =redraw(lcTemp)       
ENDIF                   
=drawCursor()           
=drawElev("HV")

RETURN .T.

****************************************************************


****************************************************************

FUNCTION moveEnd

PRIVATE lcTemp

lctemp=""               
IF lnLeftCol<lnArrDim-lnHCnt+1          
  lcTemp="TOP"          
ENDIF                   
IF lnTopRow<lnArrDim-lnVCnt+1                     
  lcTemp=lcTemp+"LEFT"  
ENDIF                   
=ClearCursor()          
lnTopRow=lnArrDim-lnVCnt+1                                   
lnLeftCol=lnArrDim-lnHCnt+1                                  
lnRow=lnVCnt                 
lnCol=lnHCnt                 
IF ! EMPTY(lcTemp)      
  =redraw(lcTemp)       
ENDIF                   
=drawCursor()           
=drawElev("HV")

RETURN .T.

****************************************************************


****************************************************************

FUNCTION screenLeft

* If cursor isn't on leftmost column of screen...            
IF lnCol>1                                                   
  * ...simply move it there...                               
  =clearCursor()                                             
  lnCol=1                                                    
  =drawCursor()                                              
  =drawElev("H")
ELSE                                                         
  * ...else move the window one screen left minus one column.
  lnLeftCol=MAX(1,lnLeftCol-lnHCnt+1)                          
  =redraw("TOP")                                             
  =drawCursor()                                              
ENDIF                                                        

RETURN .T.

****************************************************************


****************************************************************

FUNCTION screenRight

PRIVATE lnHScroll,lnVScroll,lnVMove,lcTemp

=ClearCursor()
lcTemp=""
* If cursor isn't on the rightmost column with
* data of the screen...              
IF lnCol<MIN(lnHCnt,lnTopRow+lnVCnt-lnLeftCol)
  * ...move it there, also going down in the process
  * if this is what it takes...
  lnCol=MIN(lnHCnt,lnTopRow+lnVCnt-lnLeftCol)
 	lcTemp=""
	IF lnLeftCol+lnCol-1>lnTopRow+lnRow-1
		lcTemp="V"
		lnRow=lnLeftcol+lnCol-lnTopRow
	ENDIF
  =drawCursor()                                                 
  =drawElev(lcTemp+"H")
ELSE                                                            
  * ...else if the cursor is on the rightmost column
  * of the window...
  IF lnCol=lnHCnt
		* Plan for the biggest possible horizontal scroll...
		lnHScroll=MIN(lnHCnt-1,lnArrDim-lnHCnt+1-lnLeftCol)

		* Compute how much we should move the cursor down...
		lnVMove=MAX(0,lnLeftCol+lnCol+lnHScroll-lnTopRow-lnRow)
	
		* Move the cursor down accordingly...
		IF (lnVCnt-lnRow)<lnVMove
			lnVScroll=lnVMove-(lnVCnt-lnRow)
			lnRow=lnVCnt
		ELSE
			lnVScroll=0
			lnRow=lnRow+lnVMove
		ENDIF

		* Reduce lnHScroll and lnVScroll if we have to scroll 
		* vertically by more than one window minus one line.
		IF lnVScroll>lnVCnt-1
			lnHScroll=lnHScroll-(lnVScroll-(lnVCnt-1))
			lnVScroll=lnVCnt-1
		ENDIF

		IF lnVScroll>0
			lcTemp="LEFT"
		ENDIF
		lnTopRow=lnTopRow+lnVScroll
		lnLeftCol=lnLeftCol+lnHScroll

	ELSE
		* ...just scroll the screen vertically to  get some
		* data in the right bottom corner, taking care not to
		* scroll by more than a full window minus one line.
		
		lcTemp="LEFT"
		lnHMove=MIN(lnVCnt-1,lnHCnt-lnCol)
		lnTopRow=lnTopRow+lnHMove
		lnCol=lnCol+lnHMove

	ENDIF
		
	=Redraw(lcTemp+"TOP")
	=DrawCursor()
ENDIF                                                           

RETURN .T.

****************************************************************


****************************************************************

FUNCTION screenUp

PRIVATE lnHScroll,lnVScroll,lnHMove,lcTemp

=ClearCursor()
lcTemp=""
* If cursor isn't on the topmost line with
* data of the screen...              
IF lnRow>MAX(1,lnLeftCol-LnTopRow+1)
  * ...move it there, also going left in the process
  * if this is what it takes...
  lnRow=MAX(1,lnLeftCol-LnTopRow+1)

 	lcTemp=""
	IF lnLeftCol+lnCol-1>lnTopRow+lnRow-1
		lcTemp="H"
		lnCol=lnTopRow+lnRow-lnLeftCol
	ENDIF

  =drawCursor()                                                 
  =drawElev(lcTemp+"V")

ELSE                                                            
  * ...else if the cursor is on the top row
  * of the window...
  IF lnRow=1
		* Plan for the biggest possible vertical scroll...
		lnVScroll=MIN(lnVCnt-1,LnTopRow-1)

		* Compute how much we should move the cursor left...
		lnHMove=MAX(0,lnLeftCol+lnCol-lnTopRow-lnRow+lnVScroll)
		
		* Move the cursor left accordingly...
		IF lnCol-1<lnHMove
			lnHScroll=lnHMove-lnCol+1
			lnCol=1
		ELSE
			lnHScroll=0
			lnCol=lnCol-lnHMove
		ENDIF

		* Reduce lnHScroll and lnVScroll if we have to scroll 
		* horizontally by more than one window minus one column.
		IF lnHScroll>lnHCnt-1
			lnVScroll=lnVScroll-(lnHScroll-(lnHCnt-1))
			lnHScroll=lnHCnt-1
		ENDIF

		IF lnHScroll>0
			lcTemp="TOP"
		ENDIF
		lnTopRow=lnTopRow-lnVScroll
		lnLeftCol=lnLeftCol-lnHScroll

	ELSE
		* ...just scroll the screen horizontally to  get some
		* data in the top left corner, taking care not to
		* scroll by more than a full window minus one column.
		
		lcTemp="TOP"
		lnHMove=MIN(lnHCnt-1,lnRow-1)
		lnLeftCol=lnLeftCol-lnHMove
		lnRow=lnRow-lnHMove

	ENDIF
		
	=Redraw(lcTemp+"LEFT")
	=DrawCursor()
ENDIF                                                           

RETURN .T.

****************************************************************


****************************************************************

FUNCTION screenDown

* If cursor isn't on bottom line of screen...                 
IF lnRow<lnVCnt                                               
  * ...simply move it there...                                
  =clearCursor()                                              
  lnRow=lnVCnt                                                
  =drawCursor()                                               
  =drawElev("V")
ELSE                                                          
  * ...else move the window one screen down minus one line.   
  lnTopRow=IIF((lnTopRow+lnVCnt-1)>(lnArrDim-lnVCnt+1), ;
                lnArrDim-lnVCnt+1, ;                     
                lnTopRow+lnVCnt-1)                            
  =redraw("LEFT")                                             
  =drawCursor()                                               
ENDIF                                                         

RETURN .T.

****************************************************************


****************************************************************

FUNCTION mousePress

PRIVATE lnMRow,lnMCol,lnTempRow,lnTempCol

lnMRow=MROW()
lnMCol=MCOL()

* Check exit buttons
IF lnMRow = lnVCnt+5
	* OK...
	IF BETWEEN(lnMCol,WCOLS()-20,WCOLS()-13)
		=exitAEdit("OK")
	ENDIF
	* Cancel...
	IF BETWEEN(lnMCol,WCOLS()-10,WCOLS()-3)
		=exitAEdit("Cancel")
	ENDIF
	RETURN .T.
ENDIF

* Check horizontal elevator
IF lnMRow=lnVCnt+3
	DO CASE
		* Left elevator arrow
		CASE lnMCol=lnLeftT+1
			=moveLeft()
		* Right elevator arrow
		CASE lnMCol=lnRowLen+1
			=moveRight()
		* Left elevator zone
		CASE BETWEEN(lnMCol,lnLeftT+2,lnLeftT+lnHElev)
			=screenLeft()
		* Right elevator zone
		CASE BETWEEN(lnMCol,lnLeftT+2+lnHElev,lnRowLen)
			=screenRight()
	ENDCASE
	RETURN .T.
ENDIF

* Check vertical elevator
IF lnMCol=lnRowLen+2
	DO CASE
		* Up elevator arrow
		CASE lnMRow=3
			=moveUp()
		* Down elevator arrow
		CASE lnMRow=lnVCnt+2
			=moveDown()
		* Up elevator zone
		CASE BETWEEN(lnMRow,4,3+lnVElev)
			=screenUp()
		* Down elevator zone
		CASE BETWEEN(lnMRow,4+lnVElev,lnVCnt+1)
			=screenDown()
	ENDCASE
ENDIF

* Check input zone
IF BETWEEN(lnMRow,3,lnVCnt+2)
	IF BETWEEN(lnMCol,lnLeftT+2,lnRowLen)
		lnTempRow=lnMrow-2
		lnTempCol=CEILING((lnMCol-lnLeftT-1)/lnCellLen)
		IF (lnLeftCol+lnTempCol-1)<=(lnTopRow+lnTempRow-1)
			=clearCursor()
			lnRow=lnTempRow
			lnCol=lnTempCol
			=drawElev("HV")
			=drawCursor()
			=editCell()
		ENDIF
	ENDIF
ENDIF

RETURN .T.

****************************************************************


****************************************************************

FUNCTION exitAEdit

PARAMETERS lcStat

DO CASE
	CASE lcStat="OK"
		* Simulate button press...
		@ lnVCnt+5,WCOLS()-20 FILL TO lnVCnt+5,WCOLS()-13 COLOR (SCHEME(10,2))
		WAIT WINDOW "Saving..." TIMEOUT 0.2
		* Exit...
		lcStatus="OK"
		llExit=.T.
	CASE lcStat="Cancel"
		* Simulate button press...
		@ lnVCnt+5,WCOLS()-10 FILL TO lnVCnt+5,WCOLS()-3 COLOR (SCHEME(10,2))
		WAIT WINDOW "Canceling..." TIMEOUT 0.2
		* Exit...
		lcStatus="Cancel"
		llExit=.T.
ENDCASE
	
